<?php
class Chronopost_Chronorelais_Block_Adminhtml_Notification extends Mage_Core_Block_Template
{
    const XML_SEVERITY_ICONS_URL_PATH  = 'system/adminnotification/severity_icons_url';

    const MODULE_RELEASES_XML_URL = 'http://connect20.magentocommerce.com/community/Chronopost/releases.xml';
    //all community packages => http://connect20.magentocommerce.com/community/packages.xml

    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate('chronorelais/notification.phtml');
        }
        return $this;
    }

    public function getSeverityIconsUrl()
    {
        return (Mage::app()->getFrontController()->getRequest()->isSecure() ? 'https://' : 'http://')
                . sprintf(Mage::getStoreConfig(self::XML_SEVERITY_ICONS_URL_PATH), Mage::getVersion(),
                    'SEVERITY_NOTICE');
    }

    public function canShow()
    {
        if (!Mage::getSingleton('admin/session')->isFirstPageAfterLogin()) {
            return false;
        }
        return true;
    }

    public function getNotifications()
    {
        $notifications = array();
        /* test if WS is ok */
        /** @var Chronopost_Chronorelais_Helper_Data $_helper */
        $_helper = Mage::helper('chronorelais');
        $contratDefault = $_helper->getSpecificContract(0);
        $account_number = $_helper->getConfigurationAccountNumber($contratDefault);
        $password = $_helper->getConfigurationAccountPass($contratDefault);

        $origin_postcode = $_helper->getConfigurationShipperInfo('zipcode');

        $WSParams = array(
            'accountNumber' => $account_number,
            'password' => $password,
            'depCountryCode' => $_helper->getConfigurationShipperInfo('country'),
            'depZipCode' => $origin_postcode,
            'arrCountryCode' => $_helper->getConfigurationShipperInfo('country'),
            'arrZipCode' => $origin_postcode,
            'arrCity' => $_helper->getConfigurationShipperInfo('city'),
            'type' => 'M',
            'weight' => 1
        );

        $helperWS = Mage::helper('chronorelais/webservice');
        $webservbt = $helperWS->checkLogin($WSParams);

        if(!$webservbt) {
            $notifications[] = 'quickcost_not_available';
        } else {
            $webservbt = (array)$webservbt;
            if(isset($webservbt['errorCode']) && $webservbt['errorCode'] != 0) {
                $notifications[] = 'quickcost_not_available';
            }
        }

        /* test if new version is available */
        $currentVersion = (string)Mage::getConfig()->getModuleConfig("Chronopost_Chronorelais")->version;

        $xml = simplexml_load_file(self::MODULE_RELEASES_XML_URL);
        $nbRelease = count($xml->children());
        $releases = $xml->children();
        $lastRelease = $releases[0];
        if(version_compare($lastRelease->v, $currentVersion, '>')) {
            $notifications[] = 'new_version';
        }

        if(Mage::getStoreConfig('chronorelais/retour/defaultadress') == 'return_adress' && !$this->haveReturnConfig()) {
            $notifications[] = 'not_return_information';
        }

        return $notifications;

    }

    protected function haveReturnConfig(){
        return !(is_null(Mage::getStoreConfig('chronorelais/returninformation/name')) ||
                    Mage::getStoreConfig('chronorelais/returninformation/name') == "") &&
               !(is_null(Mage::getStoreConfig('chronorelais/returninformation/address1')) ||
                        Mage::getStoreConfig('chronorelais/returninformation/address1') == "") &&
               !(is_null(Mage::getStoreConfig('chronorelais/returninformation/zipcode')) ||
                   Mage::getStoreConfig('chronorelais/returninformation/zipcode') == "") &&
               !(is_null(Mage::getStoreConfig('chronorelais/returninformation/city')) ||
                   Mage::getStoreConfig('chronorelais/returninformation/city') == "") &&
               !(is_null(Mage::getStoreConfig('chronorelais/returninformation/country')) ||
                   Mage::getStoreConfig('chronorelais/returninformation/country') == "");
    }

}
