<?php
class Chronopost_Chronorelais_Adminhtml_Chronorelais_ImportController extends Mage_Adminhtml_Controller_Action
{

    /**
     * Constructor
     */
    protected function _construct()
    {
        $this->setUsedModuleName('Chronopost_Chronorelais');
    }

    /**
     * Main action : show import form
     */
    public function indexAction()
    {
        $this->loadLayout()
            ->_setActiveMenu('sales/chronorelais/import')
            ->_addContent($this->getLayout()->createBlock('chronorelais/import_form'))
            ->renderLayout();
    }

    /**
     * Import Action
     */
    public function importAction()
    {
        $colNumberParcel = $this->getRequest()->getPost('import_chronorelais_column_number_parcel');
        $colNumberOrder= $this->getRequest()->getPost('import_chronorelais_column_number_order');

        if($colNumberParcel == null || $colNumberParcel == "" || !is_numeric($colNumberParcel)) {
            $this->_getSession()->addError($this->__('Veuillez renseigner un numéro de colonne contenant le numéro du colis.'));
            $this->_redirect('*/*/index');
            return;
        }

        if($colNumberOrder == null || $colNumberOrder  == "" || !is_numeric($colNumberOrder)) {
            $this->_getSession()->addError($this->__('Veuillez renseigner un numéro de colonne contenant le numéro de commande.'));
            $this->_redirect('*/*/index');
            return;
        }

        $colNumberParcel = $colNumberParcel - 1;
        $colNumberOrder= $colNumberOrder - 1;

        if ($this->getRequest()->isPost() && !empty($_FILES['import_chronorelais_file']['tmp_name'])) {
            try {
                $trackingTitle = $_POST['import_chronorelais_tracking_title'];
                $this->_importChronorelaisFile(
                    $_FILES['import_chronorelais_file']['tmp_name'],
                    $trackingTitle,
                    $colNumberOrder,
                    $colNumberParcel);
            }
            catch (Mage_Core_Exception $e) {
                $this->_getSession()->addError($e->getMessage());
            }
            catch (Exception $e) {
                $this->_getSession()->addError($e->getMessage());
                $this->_getSession()->addError($this->__('Invalid file upload attempt'));
            }
        }
        else {
            $this->_getSession()->addError($this->__('Invalid file upload attempt'));
        }
        $this->_redirect('*/*/index');
    }

    /**
     * Importation logic
     * @param string $fileName
     * @param string $trackingTitle
     */
    protected function _importChronorelaisFile($fileName, $trackingTitle, $colNumberOrder, $colNumberParcel)
    {
        $helper = Mage::helper('chronorelais');
        // Save config
        Mage::getConfig()->saveConfig('chronorelais/import/column_parcel', Mage::app()->getRequest()->getParam('import_chronorelais_column_number_parcel'));
        Mage::getConfig()->saveConfig('chronorelais/import/column_order', Mage::app()->getRequest()->getParam('import_chronorelais_column_number_order'));

        /**
         * File handling
         **/
        ini_set('auto_detect_line_endings', true);
        $csvObject = new Varien_File_Csv();
        $csvData = $csvObject
            ->setDelimiter(';')
            ->getData($fileName);

        /**
         * Get configuration
         */
        $sendEmail = $helper->getConfigurationSendEmail();
        $comment = $helper->getConfigurationShippingComment();
        $includeComment = $helper->getConfigurationIncludeComment();

        /**
         * $k is line number
         * $v is line content array
         */
        foreach ($csvData as $k => $v) {

            /**
             * End of file has more than one empty lines
             */
            if (count($v) <= 1 && !strlen($v[0])) {
                continue;
            }

            /**
             * Get fields content
             */
            $orderId = $v[$colNumberOrder];
            $trackingNumbers = $v[$colNumberParcel];

            if (!is_numeric($orderId) && $k == 0) {
                continue;
            }

            /**
             * Try to load the order
             */
            $order = Mage::getModel('sales/order')->loadByIncrementId($orderId);
            if (!$order->getId()) {
                $this->_getSession()->addError($this->__('Order %s does not exist', $orderId));
                continue;
            }

            /**
             * Try to create a shipment
             */
            $trackingNumbers = str_replace(array('[', ']'), '', trim($trackingNumbers));
            $trackNumbers = explode(',', $trackingNumbers);
            $shipmentId = $this->_createShipment($order, $trackNumbers, $trackingTitle, $sendEmail, $comment, $includeComment);
            if ($shipmentId) {
                $this->_getSession()->addSuccess($this->__('Shipment %s created for order %s, with tracking number %s', $shipmentId, $orderId, $trackingNumbers));
            }

        }//foreach

    }

    /**
     * Create new shipment for order
     * Inspired by Mage_Sales_Model_Order_Shipment_Api methods
     *
     * @param Mage_Sales_Model_Order $order (it should exist, no control is done into the method)
     * @param array $trackingNumbers
     * @param string $trackingTitle
     * @param booleam $email
     * @param string $comment
     * @param boolean $includeComment
     * @return int : shipment real id if creation was ok, else 0
     */
    public function _createShipment($order, $trackingNumbers, $trackingTitle, $email, $comment, $includeComment)
    {
        /**
         * Check shipment creation availability
         */
        if (!$order->canShip()) {
            $this->_getSession()->addError($this->__('Order %s can not be shipped or has already been shipped', $order->getRealOrderId()));
            return 0;
        }

        /**
         * Initialize the Mage_Sales_Model_Order_Shipment object
         */
        $convertor = Mage::getModel('sales/convert_order');
        $shipment = $convertor->toShipment($order);

        /**
         * Add the items to send
         */
        foreach ($order->getAllItems() as $orderItem) {
            if (!$orderItem->getQtyToShip()) {
                continue;
            }
            if ($orderItem->getIsVirtual()) {
                continue;
            }

            $item = $convertor->itemToShipmentItem($orderItem);
            $qty = $orderItem->getQtyToShip();
            $item->setQty($qty);

        	$shipment->addItem($item);
        }//foreach

        $shipment->register();

        /**
         * Tracking number instanciation
         */

		$_shippingMethod = explode("_",$order->getShippingMethod());
		$tracking_order = "";
		switch($_shippingMethod[0]) {
			case "chronopost":
			case "chronorelais":
			case "chronoexpress":
				$carrier_code = $_shippingMethod[0];
				$popup = 1;
				$tracking_title = $this->__('Track Your Order');
                foreach($trackingNumbers as $trackingNumber) {
                    $tracking_url = str_replace('{tracking_number}', $trackingNumber, Mage::helper('chronorelais')->getConfigurationTrackingViewUrl());
                    $tracking_order .= '<p><a title="' . $tracking_title . '" href="' . $tracking_url . '"><b>' . $tracking_title . '</b></a></p>';
                }
				break;
			default:
				$carrier_code = 'custom';
				$popup = 0;
				$tracking_order = '';
				break;
		}

		foreach ($trackingNumbers as $trackingNumber){
            $track = Mage::getModel('sales/order_shipment_track')
                ->setNumber($trackingNumber) //setTrackingNumber ?
                ->setCarrierCode($carrier_code)
                ->setTitle($trackingTitle)
                ->setPopup($popup);
            $shipment->addTrack($track);

            /**
             * Comment handling
             */
            $shipment->addComment($comment, $email && $includeComment);
        }


        /**
         * Change order status to Processing
         */
        $shipment->getOrder()->setIsInProcess(true);

        /**
         * If e-mail, set as sent (must be done before shipment object saving)
         */
        if ($email) {
            $shipment->setEmailSent(true);
        }

        try {
        	/**
             * Save the created shipment and the updated order
             */
            $shipment->save();
            $shipment->getOrder()->save();

            /**
             * Email sending
             */
            $shipment->sendEmail($email, ($includeComment ? $tracking_order.$comment : $tracking_order));
        } catch (Mage_Core_Exception $e) {
            $this->_getSession()->addError($this->__('Shipment creation error for Order %s : %s', $orderId, $e->getMessage()));
            return 0;
        }

        /**
         * Everything was ok : return Shipment real id
         */
        return $shipment->getIncrementId();
    }
}
