<?php

require_once 'Mage/Adminhtml/controllers/Sales/Order/ShipmentController.php';

class Chronopost_Chronorelais_Adminhtml_Chronorelais_Sales_ImpressionController extends Mage_Adminhtml_Sales_Order_ShipmentController
{

    protected $_trackingNumbers = '';

    /**
     * Additional initialization
     *
     */
    protected function _construct()
    {
        $this->setUsedModuleName('Chronopost_Chronorelais');
    }

    /**
     * Shipping grid
     */
    public function indexAction()
    {
        if (!extension_loaded('soap')) {
            $this->_getSession()->addError($this->__('The SOAP extension is not installed in the server. Please contact the site administrator. Sorry for inconvenience.'));

            return $this->_redirectReferer();
        }
        $cmdTestGs = Mage::helper('chronorelais')->getConfigData('chronorelais/shipping/gs_path') . " -v";
        if (shell_exec($cmdTestGs) === null) {
            $this->_getSession()->addNotice($this->__('Please install %s on your server to print mass',
                '<a href="http://www.ghostscript.com/download/" target="_blank">Ghostscript</a>'));
        }
        $this->loadLayout()
            ->_setActiveMenu('sales/chronorelais')
            ->_addContent($this->getLayout()->createBlock('chronorelais/sales_impression'))
            ->renderLayout();
    }

    /**
     * Save shipment and order in one transaction
     *
     * @param Mage_Sales_Model_Order_Shipment $shipment
     */
    protected function _saveShipment($shipment)
    {
        $shipment->getOrder()->setIsInProcess(true);
        $transactionSave = Mage::getModel('core/resource_transaction')
            ->addObject($shipment)
            ->addObject($shipment->getOrder())
            ->save();

        return $this;
    }

    /**
     * Declare headers and content file in response for file download
     *
     * @param string       $fileName
     * @param string|array $content       set to null to avoid starting output, $contentLength should be set explicitly in
     *                                    that case
     * @param string       $contentType
     * @param int          $contentLength explicit content length, if strlen($content) isn't applicable
     *
     * @return Mage_Core_Controller_Varien_Action
     */
    protected function _prepareDownloadResponse(
        $fileName,
        $content,
        $contentType = 'application/octet-stream',
        $contentLength = null
    ) {
        $session = Mage::getSingleton('admin/session');
        if ($session->isFirstPageAfterLogin()) {
            $this->_redirect($session->getUser()->getStartupPageUrl());

            return $this;
        }

        $isFile = false;
        $file = null;
        if (is_array($content)) {
            if (!isset($content['type']) || !isset($content['value'])) {
                return $this;
            }
            if ($content['type'] == 'filename') {
                $isFile = true;
                $file = $content['value'];
                $contentLength = filesize($file);
            }
        }

        $this->getResponse()
            ->setHttpResponseCode(200)
            ->setHeader('Pragma', 'public', true)
            ->setHeader('Cache-Control', 'must-revalidate, post-check=0, pre-check=0', true)
            ->setHeader('Content-type', $contentType, true)
            ->setHeader('Content-Length', is_null($contentLength) ? strlen($content) : $contentLength, true)
            ->setHeader('Content-Disposition', 'attachment; filename="' . $fileName . '"', true)
            ->setHeader('Last-Modified', date('r'), true);

        if (!is_null($content)) {
            if ($isFile) {
                $this->getResponse()->clearBody();
                $this->getResponse()->sendHeaders();

                $ioAdapter = new Varien_Io_File();
                $ioAdapter->open(array('path' => $ioAdapter->dirname($file)));
                $ioAdapter->streamOpen($file, 'r');
                while ($buffer = $ioAdapter->streamRead()) {
                    print $buffer;
                }
                $ioAdapter->streamClose();
                if (!empty($content['rm'])) {
                    $ioAdapter->rm($file);
                }

                exit(0);
            } else {
                $this->getResponse()->setBody($content);
            }
        }

        return $this;
    }

    protected function _processDownloadMass($pdf_contents)
    {

        $paths = array();
        $this->createMediaChronopostFolder();
        $indiceFile = 0;
        foreach ($pdf_contents as $pdf_content) {
            $fileName = 'tmp-etiquette-' . date('H-i-s-' . $indiceFile);
            /* save pdf file */
            $path = Mage::getBaseDir('media') . '/chronopost/' . $fileName . '.pdf';
            file_put_contents($path, $pdf_content);
            $paths[] = $path;
            $indiceFile++;
        }

        /* creation d'un pdf unique */
        $pdfMergeFileName = "merged-" . date('YmdHis') . ".pdf";
        $pathMerge = Mage::getBaseDir('media') . "/chronopost/" . $pdfMergeFileName;
        $cmd = Mage::helper('chronorelais')->getConfigData('chronorelais/shipping/gs_path') . ' -dNOPAUSE -sDEVICE=pdfwrite -sOutputFile="' . $pathMerge . '" -dBATCH ' . implode(' ',
                $paths);
        $res_shell = shell_exec($cmd);

        /* suppression des pdf temp */
        foreach ($paths as $path) {
            if (is_file($path)) {
                unlink($path);
            }
        }

        if ($res_shell === null) {
            return $this->_redirectReferer();
        } else {
            $this->_prepareDownloadResponse($pdfMergeFileName, array(
                'type'  => 'filename',
                'value' => $pathMerge
            ));
            unlink($pathMerge);
        }
    }

    protected function getTrackingNumber($shipmentId)
    {
        $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);

        //On récupère le numéro de tracking
        $tracks = $shipment->getTracksCollection();
        foreach ($tracks as $track) {
            if ($track->getParentId() == $shipmentId) {
                $this->_trackingNumbers .= $track->getnumber();
            }
        }

        return $this->_trackingNumbers;
    }

    protected function getFilledValue($value)
    {
        if ($value) {
            return $this->removeaccents(trim($value));
        } else {
            return '';
        }
    }

    protected function checkMobileNumber($value)
    {
        if ($reqvalue = trim($value)) {
            $_number = substr($reqvalue, 0, 2);
            $fixed_array = array('01', '02', '03', '04', '05', '06', '06');
            if (in_array($_number, $fixed_array)) {
                return $reqvalue;
            } else {
                return '';
            }
        }
    }

    protected function getExpeditionParams($shipment, $_shippingMethod, $contract = null, $dimensions = null)
    {
        $_order = $shipment->getOrder();
        $_shippingAddress = $shipment->getShippingAddress();
        $_billingAddress = $shipment->getBillingAddress();
        /** @var Chronopost_Chronorelais_Helper_Data $_helper */
        $_helper = Mage::helper('chronorelais');

        if(is_array($contract)) {
            $_contract["number"] = $contract["contract_account_number"];
            $_contract["subAccount"] = $contract["contract_sub_account_number"];
            $_contract["pass"] = $contract["contract_account_password"];
        } else {
            $_contract = ($contract != null && (int)$contract >= 0) ? $_helper->getSpecificContract($contract) : null;
        }
        $dimensions = (is_null($dimensions)) ? [] : $dimensions;

        $accountNumber = $_helper->getConfigurationAccountNumber($_contract);
        $subAccount = $_helper->getConfigurationSubAccountNumber($_contract);
        $pass = $_helper->getConfigurationAccountPass($_contract);

        $shippingMethodAllow = array(
            'chronorelais',
            'chronopost',
            'chronoexpress',
            'chronopostc10',
            'chronopostc18',
            'chronopostcclassic',
            'chronorelaiseurope',
            'chronorelaisdom',
            'chronopostsrdv',
            'chronopostsameday',
            'chronopostdimanche'
        );
        if (in_array($_shippingMethod[0], $shippingMethodAllow)) {
            $esdParams = $header = $shipper = $customer = $recipient = $ref = $skybill = $skybillParams = $password = array();

            //header parameters
            $header = array(
                'idEmit'        => 'MAG',
                'accountNumber' => $accountNumber,
                'subAccount'    => $subAccount
            );

            //shipper parameters
            $shipperMobilePhone = $this->checkMobileNumber($_helper->getConfigurationShipperInfo('mobilephone'));
            $shipper = array(
                'shipperAdress1'     => $_helper->getConfigurationShipperInfo('address1'),
                'shipperAdress2'     => $_helper->getConfigurationShipperInfo('address2'),
                'shipperCity'        => $_helper->getConfigurationShipperInfo('city'),
                'shipperCivility'    => $_helper->getConfigurationShipperInfo('civility'),
                'shipperContactName' => $_helper->getConfigurationShipperInfo('contactname'),
                'shipperCountry'     => $_helper->getConfigurationShipperInfo('country'),
                'shipperEmail'       => $_helper->getConfigurationShipperInfo('email'),
                'shipperMobilePhone' => $shipperMobilePhone,
                'shipperName'        => $_helper->getConfigurationShipperInfo('name'),
                'shipperName2'       => $_helper->getConfigurationShipperInfo('name2'),
                'shipperPhone'       => $_helper->getConfigurationShipperInfo('phone'),
                'shipperPreAlert'    => '',
                'shipperZipCode'     => $_helper->getConfigurationShipperInfo('zipcode')
            );

            //customer parameters
            $customerMobilePhone = $this->checkMobileNumber($_helper->getConfigurationCustomerInfo('mobilephone'));
            $customer = array(
                'customerAdress1'     => $_helper->getConfigurationCustomerInfo('address1'),
                'customerAdress2'     => $_helper->getConfigurationCustomerInfo('address2'),
                'customerCity'        => $_helper->getConfigurationCustomerInfo('city'),
                'customerCivility'    => $_helper->getConfigurationCustomerInfo('civility'),
                'customerContactName' => $_helper->getConfigurationCustomerInfo('contactname'),
                'customerCountry'     => $_helper->getConfigurationCustomerInfo('country'),
                'customerEmail'       => $_helper->getConfigurationCustomerInfo('email'),
                'customerMobilePhone' => $customerMobilePhone,
                'customerName'        => $_helper->getConfigurationCustomerInfo('name'),
                'customerName2'       => $_helper->getConfigurationCustomerInfo('name2'),
                'customerPhone'       => $_helper->getConfigurationCustomerInfo('phone'),
                'customerPreAlert'    => '',
                'customerZipCode'     => $_helper->getConfigurationCustomerInfo('zipcode')
            );

            //recipient parameters
            $recipient_address = $_shippingAddress->getStreet();
            if (!isset($recipient_address[1])) {
                $recipient_address[1] = '';
            }
            $customer_email = ($_shippingAddress->getEmail()) ? $_shippingAddress->getEmail() : ($_billingAddress->getEmail() ? $_billingAddress->getEmail() : $_order->getCustomerEmail());
            $recipientMobilePhone = $this->checkMobileNumber($_shippingAddress->getTelephone());
            $recipientName = $this->getFilledValue($_shippingAddress->getCompany()); //RelayPoint Name if chronorelais or Companyname if chronopost and
            $recipientName2 = $this->getFilledValue($_shippingAddress->getFirstname() . ' ' . $_shippingAddress->getLastname());
            //remove any alphabets in phone number

            $recipientPhone = trim(preg_replace("/[^0-9\.\-]/", " ", $_shippingAddress->getTelephone()));

            $recipient = array(
                'recipientAdress1'     => substr($this->getFilledValue($recipient_address[0]), 0, 38),
                'recipientAdress2'     => substr($this->getFilledValue($recipient_address[1]), 0, 38),
                'recipientCity'        => $this->getFilledValue($_shippingAddress->getCity()),
                'recipientContactName' => $recipientName2,
                'recipientCountry'     => $this->getFilledValue($_shippingAddress->getCountryId()),
                'recipientEmail'       => $customer_email,
                'recipientMobilePhone' => $recipientMobilePhone,
                'recipientName'        => $recipientName,
                'recipientName2'       => $recipientName2,
                'recipientPhone'       => $recipientPhone,
                'recipientPreAlert'    => '',
                'recipientZipCode'     => $this->getFilledValue($_shippingAddress->getPostcode()),
            );

            //ref parameters
            $recipientRef = $this->getFilledValue($_shippingAddress->getWRelayPointCode());
            if (!$recipientRef) {
                $recipientRef = $_order->getCustomerId();
            }
            $shipperRef = $_order->getRealOrderId();

            $ref = array(
                'recipientRef' => $recipientRef,
                'shipperRef'   => $shipperRef
            );

            //skybill parameters
            /* Livraison Samedi (Delivery Saturday) field */
            $SaturdayShipping = 0; //default value for the saturday shipping
            if ($_shippingMethod[0] == "chronopost" || $_shippingMethod[0] == "chronorelais" || $_shippingMethod[0] == "chronorelaisdom") {
                if (!$_deliver_on_saturday = Mage::helper('chronorelais')->getLivraisonSamediStatus($_order->getEntityId())) {
                    $_deliver_on_saturday = Mage::helper('chronorelais')->getConfigData('carriers/' . $_shippingMethod[0] . '/deliver_on_saturday');
                } else {
                    if ($_deliver_on_saturday == 'Yes') {
                        $_deliver_on_saturday = 1;
                    } else {
                        $_deliver_on_saturday = 0;
                    }
                }
                $is_sending_day = Mage::helper('chronorelais')->isSendingDay();

                if ($_shippingMethod[0] == "chronorelaisdom") {
                    if ($_deliver_on_saturday && $is_sending_day) {
                        $SaturdayShipping = 369;
                    } else {
                        $SaturdayShipping = 368;
                    }
                } else {
                    if ($_deliver_on_saturday && $is_sending_day) {
                        $SaturdayShipping = 6;
                    } elseif (!$_deliver_on_saturday && $is_sending_day) {
                        $SaturdayShipping = 1;
                    }
                }

            }

            $weight = 0;
            foreach ($shipment->getItemsCollection() as $item) {
                $weight += $item->weight * $item->qty;
            }
            if ($_helper->getConfigWeightUnit() == 'g') {
                $weight = $weight / 1000; /* conversion g => kg */
            }

            /* si chronorelaiseurope : service : 337 si poids < 3kg ou 338 si > 3kg */
            if ($_shippingMethod[0] == "chronorelaiseurope") {
                $weight <= 3 ? $SaturdayShipping = '337' : $SaturdayShipping = '338';
            }

            if ($_shippingMethod[0] == "chronopostdimanche") {
                $SaturdayShipping = Chronopost_Chronorelais_Helper_Data::CHRONOPOST_DIM_BAL_SERVICE;
            }


            $weight = 0; /* On met le poids à 0 car les colis sont pesé sur place */

            $skybill = array(
                'codCurrency'     => 'EUR',
                'codValue'        => '',
                'content1'        => '',
                'content2'        => '',
                'content3'        => '',
                'content4'        => '',
                'content5'        => '',
                'customsCurrency' => 'EUR',
                'customsValue'    => '',
                'evtCode'         => 'DC',
                'insuredCurrency' => 'EUR',
                'insuredValue'    => Mage::helper('chronorelais')->getOrderAdValorem($_order),
                'objectType'      => 'MAR',
                'productCode'     => $_helper->getChronoProductCodeToShipment($_shippingMethod[0]),
                'service'         => $SaturdayShipping,
                'shipDate'        => date('c'),
                'shipHour'        => date('H'),
                'weight'          => $dimensions[0]['weight'],
                'weightUnit'      => 'KGM',
                'height'          => $dimensions[0]['height'],
                'length'          => $dimensions[0]['length'],
                'width'           => $dimensions[0]['width']
            );

            $as = $_helper->getChronoAsCode($_shippingMethod[0]);
            if($as != ''){
                $skybill['as'] = $as;
            }

            $skybillParams = array(
                'mode' => $_helper->getConfigurationSkybillParam(),
                'withReservation' => 0
            );

            $expeditionArray = array(
                'headerValue'        => $header,
                'shipperValue'       => $shipper,
                'customerValue'      => $customer,
                'recipientValue'     => $recipient,
                'refValue'           => $ref,
                'skybillValue'       => $skybill,
                'skybillParamsValue' => $skybillParams,
                'password'           => $pass
            );

            /* si chronopostsrdv : ajout parametres supplementaires */
            if ($_shippingMethod[0] == "chronopostsrdv") {

                $chronopostsrdv_creneaux_info = $_shippingAddress->getData('chronopostsrdv_creneaux_info');
                $chronopostsrdv_creneaux_info = json_decode($chronopostsrdv_creneaux_info, true);

                $_dateRdvStart = new DateTime($chronopostsrdv_creneaux_info['deliveryDate']);
                $_dateRdvStart->setTime($chronopostsrdv_creneaux_info['startHour'],
                    $chronopostsrdv_creneaux_info['startMinutes']);

                $_dateRdvEnd = new DateTime($chronopostsrdv_creneaux_info['deliveryDate']);
                $_dateRdvEnd->setTime($chronopostsrdv_creneaux_info['endHour'],
                    $chronopostsrdv_creneaux_info['endMinutes']);


                $scheduledValue = array(
                    'appointmentValue' => array(
                        'timeSlotStartDate'   => $_dateRdvStart->format("Y-m-d") . "T" . $_dateRdvStart->format("H:i:s"),
                        'timeSlotEndDate'     => $_dateRdvEnd->format("Y-m-d") . "T" . $_dateRdvEnd->format("H:i:s"),
                        'timeSlotTariffLevel' => $chronopostsrdv_creneaux_info['tariffLevel']
                    )
                );
                $expeditionArray['scheduledValue'] = $scheduledValue;

                /* modification productCode et service car dynamique pour ce mode de livraison */

                $expeditionArray['skybillValue']['productCode'] = $chronopostsrdv_creneaux_info['productCode'];
                $expeditionArray['skybillValue']['service'] = $chronopostsrdv_creneaux_info['serviceCode'];
                if($chronopostsrdv_creneaux_info['dayOfWeek'] == 7) {
                    $expeditionArray['skybillValue']['as'] = $chronopostsrdv_creneaux_info['asCode'];
                }
            }

            return $expeditionArray;
        }
    }

    protected function getExpeditionParamsMultiParcel($shipment, $_shippingMethod, $contract = null, $nbParcel = null, $dimensions = null)
    {
        $_order = $shipment->getOrder();
        $_shippingAddress = $shipment->getShippingAddress();
        $_billingAddress = $shipment->getBillingAddress();
        /** @var Chronopost_Chronorelais_Helper_Data $_helper */
        $_helper = Mage::helper('chronorelais');
        $dimensions = (is_null($dimensions)) ? [] : $dimensions;
        if(is_array($contract)) {
            $_contract["number"] = $contract["contract_account_number"];
            $_contract["subAccount"] = $contract["contract_sub_account_number"];
            $_contract["pass"] = $contract["contract_account_password"];
        } else {
            $_contract = ($contract != null && (int)$contract >= 0) ? $_helper->getSpecificContract($contract) : null;
        }

        $accountNumber = $_helper->getConfigurationAccountNumber($_contract);
        $subAccount = $_helper->getConfigurationSubAccountNumber($_contract);
        $pass = $_helper->getConfigurationAccountPass($_contract);

        $shippingMethodAllow = array(
            'chronorelais',
            'chronopost',
            'chronoexpress',
            'chronopostc10',
            'chronopostc18',
            'chronopostcclassic',
            'chronorelaiseurope',
            'chronorelaisdom',
            'chronopostsrdv',
            'chronopostsameday',
            'chronopostdimanche'
        );
        if (in_array($_shippingMethod[0], $shippingMethodAllow)) {
            $header = $shipper = $customer = $recipient = $ref = $skybill = $skybillParams = $password = array();

            //header parameters
            $header = array(
                'idEmit'        => 'MAG',
                'accountNumber' => $accountNumber,
                'subAccount'    => $subAccount
            );

            //shipper parameters
            $shipper = array(
                'shipperAdress1' => $_helper->getConfigurationShipperInfo('address1'),
                'shipperAdress2' => $_helper->getConfigurationShipperInfo('address2'),
                'shipperCity' => $_helper->getConfigurationShipperInfo('city'),
                'shipperCivility' => $_helper->getConfigurationShipperInfo('civility'),
                'shipperContactName' => $_helper->getConfigurationShipperInfo('contactname'),
                'shipperCountry' => $_helper->getConfigurationShipperInfo('country'),
                'shipperCountryName' => Mage::app()->getLocale()->getCountryTranslation($_helper->getConfigurationShipperInfo('country')),
                'shipperEmail' => $_helper->getConfigurationShipperInfo('email'),
                'shipperName' => $_helper->getConfigurationShipperInfo('name'),
                'shipperName2' => $_helper->getConfigurationShipperInfo('name2'),
                'shipperPhone' => $_helper->getConfigurationShipperInfo('phone'),
                'shipperPreAlert' => '',
                'shipperZipCode' => $_helper->getConfigurationShipperInfo('zipcode')
            );

            //customer parameters
            $customer = array(
                'customerAdress1' => $_helper->getConfigurationCustomerInfo('address1'),
                'customerAdress2' => $_helper->getConfigurationCustomerInfo('address2'),
                'customerCity' => $_helper->getConfigurationCustomerInfo('city'),
                'customerCivility' => $_helper->getConfigurationCustomerInfo('civility'),
                'customerContactName' => $_helper->getConfigurationCustomerInfo('contactname'),
                'customerCountry' => $_helper->getConfigurationCustomerInfo('country'),
                'customerCountryName' => Mage::app()->getLocale()->getCountryTranslation($_helper->getConfigurationCustomerInfo('country')),
                'customerEmail' => $_helper->getConfigurationCustomerInfo('email'),
                'customerName' => $_helper->getConfigurationCustomerInfo('name'),
                'customerName2' => $_helper->getConfigurationCustomerInfo('name2'),
                'customerPhone' => $_helper->getConfigurationCustomerInfo('phone'),
                'customerPreAlert' => '',
                'customerZipCode' => $_helper->getConfigurationCustomerInfo('zipcode')
            );

            //recipient parameters
            $recipient_address = $_shippingAddress->getStreet();
            if (!isset($recipient_address[1])) {
                $recipient_address[1] = '';
            }
            $customer_email = ($_shippingAddress->getEmail()) ? $_shippingAddress->getEmail() : ($_billingAddress->getEmail() ? $_billingAddress->getEmail() : $_order->getCustomerEmail());
            $recipientName = $this->getFilledValue($_shippingAddress->getCompany()); //RelayPoint Name if chronorelais or Companyname if chronopost and
            $recipientName2 = $this->getFilledValue($_shippingAddress->getFirstname() . ' ' . $_shippingAddress->getLastname());
            //remove any alphabets in phone number

            $recipientPhone = trim(preg_replace("/[^0-9\.\-]/", " ", $_shippingAddress->getTelephone()));

            $recipient = array(
                'recipientAdress1' => substr($this->getFilledValue($recipient_address[0]), 0, 38),
                'recipientAdress2' => substr($this->getFilledValue($recipient_address[1]), 0, 38),
                'recipientCity' => $this->getFilledValue($_shippingAddress->getCity()),
                'recipientContactName' => $recipientName2,
                'recipientCountry' => $this->getFilledValue($_shippingAddress->getCountryId()),
                'recipientCountryName' => Mage::app()->getLocale()->getCountryTranslation($this->getFilledValue($_shippingAddress->getCountryId())),
                'recipientEmail' => $customer_email,
                'recipientName' => $recipientName,
                'recipientName2' => $recipientName2,
                'recipientPhone' => $recipientPhone,
                'recipientPreAlert' => '',
                'recipientZipCode' => $this->getFilledValue($_shippingAddress->getPostcode()),
            );

            //ref parameters
            $refValue = array();

            for($i = 0; $i < $nbParcel ; $i++) {
                $recipientRef = $this->getFilledValue($_shippingAddress->getWRelayPointCode());
                if (!$recipientRef) {
                    $recipientRef = $_order->getId();
                }
                $shipperRef = $_order->getRealOrderId();

                array_push($refValue, array('recipientRef' => $recipientRef,
                                            'shipperRef' => $shipperRef));
            }

            $skybillValue = array();

            for($i = 1; $i <= $nbParcel; $i++) {
                $dimension = $dimensions[$i-1];
                $_skybill = array(
                    'bulkNumber' => $nbParcel,
                    'skybillRank' => $i,
                    'evtCode' => 'DC',
                    'objectType' => 'MAR',
                    'productCode' => $_helper->getChronoProductCodeToShipment($_shippingMethod[0]),
                    'service' => $_helper->getChronoServiceProductCode($_shippingMethod[0]),
                    'shipDate' => date('c'),
                    'shipHour' => date('H'),
                    'weight' => $dimension['weight'],
                    'weightUnit' => 'KGM',
                    'height'          => $dimension['height'],
                    'length'          => $dimension['length'],
                    'width'           => $dimension['width']
                );

                if ($_shippingMethod[0] == "chronopostsrdv") {
                    $chronopostsrdv_creneaux_info = $_shippingAddress->getData('chronopostsrdv_creneaux_info');
                    $chronopostsrdv_creneaux_info = json_decode($chronopostsrdv_creneaux_info, true);
                    $_skybill['productCode'] = $chronopostsrdv_creneaux_info['productCode'];
                    $_skybill['service'] = $chronopostsrdv_creneaux_info['serviceCode'];
                    if ($chronopostsrdv_creneaux_info['dayOfWeek'] == 7) {
                        $_skybill['as'] = $chronopostsrdv_creneaux_info['asCode'];
                    }
                }

                array_push($skybillValue, $_skybill);
            }

            $scheduledValue = [];

            if ($_shippingMethod[0] == "chronopostsrdv") {
                    $chronopostsrdv_creneaux_info = $_shippingAddress->getData('chronopostsrdv_creneaux_info');
                    $chronopostsrdv_creneaux_info = json_decode($chronopostsrdv_creneaux_info, true);

                    $_dateRdvStart = new DateTime($chronopostsrdv_creneaux_info['deliveryDate']);
                    $_dateRdvStart->setTime($chronopostsrdv_creneaux_info['startHour'],
                        $chronopostsrdv_creneaux_info['startMinutes']);

                    $_dateRdvEnd = new DateTime($chronopostsrdv_creneaux_info['deliveryDate']);
                    $_dateRdvEnd->setTime($chronopostsrdv_creneaux_info['endHour'],
                        $chronopostsrdv_creneaux_info['endMinutes']);

                    $scheduledValue['appointmentValue'] = [
                        'timeSlotStartDate'   => $_dateRdvStart->format("Y-m-d") . "T" . $_dateRdvStart->format("H:i:s"),
                        'timeSlotEndDate'     => $_dateRdvEnd->format("Y-m-d") . "T" . $_dateRdvEnd->format("H:i:s"),
                        'timeSlotTariffLevel' => $chronopostsrdv_creneaux_info['tariffLevel']
                    ];
            }


            $skybillParamsValue = array(
                'mode' => $_helper->getConfigurationSkybillParam(),
            );

            $expeditionArray = array(
                'headerValue' => $header,
                'shipperValue' => $shipper,
                'customerValue' => $customer,
                'recipientValue' => $recipient,
                'refValue' => $refValue,
                'skybillValue' => $skybillValue,
                'skybillParamsValue' => $skybillParamsValue,
                'scheduledValue' => $scheduledValue,
                'password' => $pass,
                'version' => '3.0',
                'numberOfParcel' => $nbParcel
            );

            return $expeditionArray;
        }
    }

    protected function getEtiquetteUrl($shipmentId, $contractId = null, $dimensions = null)
    {
        //On récupère les infos d'expédition
        /** @var Chronopost_Chronorelais_Helper_Data $_helper */
        $_helper = Mage::helper('chronorelais');
        $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);
        $_order = $shipment->getOrder();
        $_shippingMethod = explode("_", $_order->getShippingMethod());
        if($dimensions == null) {
            $dimensions = array();
        }

        /*if ($_shipTracks = $shipment->getAllTracks()) {
            $skybill = [];
            foreach ($_shipTracks as $_shipTrack) {
                if ($_shipTrack->getNumber() && $_shipTrack->getChronoReservationNumber()) {
                    $pdf = base64_decode($_shipTrack->getChronoReservationNumber());
                    if(!in_array($pdf, $skybill)) {
                        $skybill[] = $pdf;
                    }
                    //break;
                }
            }
            if (count($skybill) > 0) {
                return count($skybill) > 1 ? $skybill : $skybill[0];
            }
        }*/

        if($contractId < 0) {
            $contract = $_helper->getContractOfOrder($_order->getId());
            $expeditionArray = $this->getExpeditionParams($shipment, $_shippingMethod, $contract, $dimensions);
        } else {
            $expeditionArray = $this->getExpeditionParams($shipment, $_shippingMethod, $contractId, $dimensions);
        }

        if ($expeditionArray) {
            $client = new SoapClient("https://www.chronopost.fr/shipping-cxf/ShippingServiceWS?wsdl",
                array('trace' => true));
            try {
                $webservbt = $client->shippingV7($expeditionArray);
                if (!$webservbt->return->errorCode && $webservbt->return->pdfEtiquette) {
                    $tracking_number = $webservbt->return->skybillNumber;
                    if(!$_helper->saveContractOfOrder($_order->getId(), $contractId)) {
                        $this->_getSession()->addError($_helper->__("Invalid Contract"));
                        return;
                    };
                    // Add tracking number for the shipment if not already exists.
                    if (!$this->_trackingNumbers && $webservbt->return->skybillNumber) {
                        $track = Mage::getModel('sales/order_shipment_track')
                            ->setNumber($webservbt->return->skybillNumber)
                            ->setChronoReservationNumber(base64_encode($webservbt->return->pdfEtiquette))
                            ->setCarrier(ucwords($_shippingMethod[0]))
                            ->setCarrierCode($_shippingMethod[0])
                            ->setTitle(ucwords($_shippingMethod[0]))
                            ->setPopup(1);
                        $shipment->addTrack($track);
                        Mage::getModel('chronorelais/historyLt')->saveLtHistory($shipmentId, $webservbt->return->skybillNumber, $dimensions[0]['weight']);

                        $tracking_url = str_replace('{tracking_number}', $tracking_number,
                            Mage::helper('chronorelais')->getConfigurationTrackingViewUrl());
                        $tracking_title = $this->__('Track Your Order');
                        $tracking_order = '<p><a title="' . $tracking_title . '" href="' . $tracking_url . '"><b>' . $tracking_title . '</b></a></p>';

                        $comment = '';
                        $shipment->setEmailSent(true);
                        $this->_saveShipment($shipment);
                        //$shipment->sendEmail(1, $tracking_order . $comment);
                    }

                    return $webservbt->return->pdfEtiquette; // pdf base64
                } else {
                    $errorMessage = $_helper->__($webservbt->return->errorMessage);
                    if($webservbt->return->errorCode == 1) {
                        $errorMessage = $_helper->__("Le colis de la commande " . $_order->getIncrementId() . " n'appartient pas au contrat passé en paramètre.");
                    }
                    $this->_getSession()->addError($errorMessage);
                }
            } catch (SoapFault $fault) {
                $this->_getSession()->addError($_helper->__($fault->faultstring));
            }
        }
    }

    protected function getEtiquetteMultiParcel($shipmentId, $contractId = null, $nbParcel = null, $dimensions = null)
    {
        //On récupère les infos d'expédition
        /** @var Chronopost_Chronorelais_Helper_Data $_helper */
        $_helper = Mage::helper('chronorelais');
        $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);
        $_order = $shipment->getOrder();
        $_shippingMethod = explode("_", $_order->getShippingMethod());

        if($dimensions == null) {
            $dimensions = array();
        }

        if($contractId < 0) {
            $contract = $_helper->getContractOfOrder($_order->getId());
            $expeditionArray = $this->getExpeditionParamsMultiParcel($shipment, $_shippingMethod, $contract, $nbParcel, $dimensions);
        } else {
            $expeditionArray = $this->getExpeditionParamsMultiParcel($shipment, $_shippingMethod, $contractId, $nbParcel, $dimensions);
        }

        if ($expeditionArray) {
            $client = new SoapClient("https://www.chronopost.fr/shipping-cxf/ShippingServiceWS?wsdl",
                array('trace' => true));
            try {
                $webservbt = $client->shippingMultiParcelWithReservationV3($expeditionArray);
                if (!$webservbt->return->errorCode && $webservbt->return->resultParcelValue && $webservbt->return->reservationNumber) {
                    $parcelValues = $webservbt->return->resultParcelValue;
                    $reservationNumber = $webservbt->return->reservationNumber;
                    $skybillReservation = $client->getReservedSkybill(array('reservationNumber' => $reservationNumber));

                    if(!$skybillReservation->return->errorCode && $skybillReservation->return->skybill){
                        $pdfEtiquette = $skybillReservation->return->skybill;
                        // Add tracking number for the shipment if not already exists.
                        if (!$this->_trackingNumbers && count($parcelValues) >= 1) {
                            $tracking_order = '';
                            $cpt = 0;
                            foreach ($parcelValues as $i => $parcel) {
                                $track = Mage::getModel('sales/order_shipment_track')
                                    ->setWeight($dimensions[$i])
                                    ->setNumber($parcel->skybillNumber)
                                    ->setChronoReservationNumber($pdfEtiquette)
                                    ->setCarrier(ucwords($_shippingMethod[0]))
                                    ->setCarrierCode($_shippingMethod[0])
                                    ->setTitle(ucwords($_shippingMethod[0]))
                                    ->setPopup(1);
                                $shipment->addTrack($track);

                                Mage::getModel('chronorelais/historyLt')->saveLtHistory($shipmentId, $parcel->skybillNumber, $dimensions[$cpt]['weight']);

                                $tracking_number = $parcel->skybillNumber;
                                $tracking_url = str_replace('{tracking_number}', $tracking_number,
                                    Mage::helper('chronorelais')->getConfigurationTrackingViewUrl());
                                $tracking_title = $this->__('Track Your Order');
                                $tracking_order .= '<p><a title="' . $tracking_title . '" href="' . $tracking_url . '"><b>' . $tracking_title . '</b></a></p>';
                                $cpt++;
                            }

                            $comment = '';
                            $shipment->setEmailSent(true);
                            $this->_saveShipment($shipment);
                            $shipment->sendEmail(1, $tracking_order . $comment);
                        }
                        $pdfEtiquette = base64_decode($pdfEtiquette);
                        return $pdfEtiquette;
                    } else {
                        $this->_getSession()->addError($skybillReservation->return->errorMessage);
                    }
                } else {
                    $errorMessage = $_helper->__($webservbt->return->errorMessage);
                    if($webservbt->return->errorCode == 1) {
                        $errorMessage = $_helper->__("Le colis de la commande " . $_order->getIncrementId() . " n'appartient pas au contrat passé en paramètre.");
                    }
                    $this->_getSession()->addError($errorMessage);
                }
            } catch (SoapFault $fault) {
                $this->_getSession()->addError($_helper->__($fault->faultstring));
            }
        }
    }

    public function getShipmentByOrderId($orderId)
    {
        $_shipment = Mage::getResourceModel('sales/order_shipment_grid_collection')
            ->addAttributeToFilter('order_id', $orderId)
            ->getAllIds();

        return $_shipment;
    }

    public function getShipmentByIncrementId($incrementId)
    {
        $_shipment = Mage::getResourceModel('sales/order_shipment_grid_collection')
            ->addAttributeToFilter('increment_id', $incrementId)
            ->getAllIds();

        return $_shipment;
    }

    public function initShipment($orderId, $savedQtys = '')
    {
        $order = Mage::getModel('sales/order')->load($orderId);

        /**
         * Check order existing
         */
        if (!$order->getId()) {
            $this->_getSession()->addError($this->__('The order no longer exists.'));

            return false;
        }
        /**
         * Check shipment is available to create separate from invoice
         */
        if ($order->getForcedDoShipmentWithInvoice()) {
            $this->_getSession()->addError($this->__('Cannot do shipment for the order separately from invoice.'));

            return false;
        }
        /**
         * Check shipment create availability
         */
        if (empty($savedQtys)) {
            $savedQtys = $this->_getItemQtys();
        }
        $shipment = Mage::getModel('sales/service_order', $order)->prepareShipment($savedQtys);
        if (Mage::registry('current_shipment')) {
            Mage::unregister('current_shipment');
        }
        Mage::register('current_shipment', $shipment);

        return $shipment;
    }

    public function createNewShipment($orderId, $savedQtys = '', $contractId = null, $number_parcel = 0, $dimensions = null)
    {
        $_helper = Mage::helper('chronorelais');
        $skybill = '';

        if($dimensions == null) {
            $dimensions = array();
        }

        try {
            if ($shipment = $this->initShipment($orderId, $savedQtys)) {
                $shipment->register();

                $_order = $shipment->getOrder();
                $_shippingMethod = explode("_", $_order->getShippingMethod());

                if($number_parcel > 1){
                    $expeditionArray = $this->getExpeditionParamsMultiParcel($shipment, $_shippingMethod, $contractId, $number_parcel, $dimensions);
                } else {
                    $expeditionArray = $this->getExpeditionParams($shipment, $_shippingMethod, $contractId, $dimensions);
                }

                $tracking_number = '';
                if ($expeditionArray) {
                    $client = new SoapClient("https://www.chronopost.fr/shipping-cxf/ShippingServiceWS?wsdl",
                        array('trace' => true));
                    try {
                        if($number_parcel > 1) {
                            $expedition = $client->shippingMultiParcelWithReservationV3($expeditionArray);

                            if (!$expedition->return->errorCode && $expedition->return->resultParcelValue && $expedition->return->reservationNumber) {
                                $parcelValues = $expedition->return->resultParcelValue;
                                $reservationNumber = $expedition->return->reservationNumber;
                                $skybillReservation = $client->getReservedSkybill(array('reservationNumber' => $reservationNumber));
                                if(!$skybillReservation->return->errorCode && $skybillReservation->return->skybill){
                                    if(!$_helper->saveContractOfOrder($_order->getId(), $contractId)) {
                                        $this->_getSession()->addError($_helper->__("Invalid Contract"));
                                        $this->_redirect('*/*/new', array('order_id' => $this->getRequest()->getParam('order_id')));
                                        return;
                                    };
                                    $skybill = $skybillReservation->return->skybill;
                                    $tracking_order = '';
                                    $cpt = 0;
                                    foreach ($parcelValues as $i => $parcel) {
                                        $track = Mage::getModel('sales/order_shipment_track')
                                            ->setWeight($dimensions[$i]['weight'])
                                            ->setNumber($parcel->skybillNumber)
                                            ->setChronoReservationNumber($skybill)
                                            ->setCarrier(ucwords($_shippingMethod[0]))
                                            ->setCarrierCode($_shippingMethod[0])
                                            ->setTitle(ucwords($_shippingMethod[0]))
                                            ->setPopup(1);
                                        $shipment->addTrack($track);

                                        Mage::getModel('chronorelais/historyLt')->saveLtHistory($shipment->getId(), $parcel->skybillNumber, $dimensions[$cpt]['weight']);

                                        $tracking_number = $parcel->skybillNumber;

                                        $tracking_url = str_replace('{tracking_number}', $tracking_number,
                                            Mage::helper('chronorelais')->getConfigurationTrackingViewUrl());
                                        $tracking_title = $this->__('Track Your Order');
                                        $tracking_order .= '<p><a title="' . $tracking_title . '" href="' . $tracking_url . '"><b>' . $tracking_title . '</b></a></p>';
                                        $cpt++;
                                    }
                                    $skybill = base64_decode($skybill);
                                } else {
                                    $this->_getSession()->addError($_helper->__($skybillReservation->return->errorMessage));
                                    $this->_redirect('*/*/new', array('order_id' => $this->getRequest()->getParam('order_id')));
                                    return;
                                }
                            } else {
                                $this->_getSession()->addError($_helper->__($expedition->return->errorMessage));
                                $this->_redirect('*/*/new', array('order_id' => $this->getRequest()->getParam('order_id')));
                                return;
                            }
                        } else {
                            $expedition = $client->shippingV7($expeditionArray);
                            if (!$expedition->return->errorCode && $expedition->return->skybillNumber) {
                                if(!$_helper->saveContractOfOrder($_order->getId(), $contractId)) {
                                    $this->_getSession()->addError($_helper->__("Invalid Contract"));
                                    $this->_redirect('*/*/new', array('order_id' => $this->getRequest()->getParam('order_id')));
                                    return;
                                };
                                $tracking_number = $expedition->return->skybillNumber;
                                $track = Mage::getModel('sales/order_shipment_track')
                                    ->setNumber($expedition->return->skybillNumber)
                                    ->setChronoReservationNumber(base64_encode($expedition->return->pdfEtiquette))
                                    ->setCarrier(ucwords($_shippingMethod[0]))
                                    ->setCarrierCode($_shippingMethod[0])
                                    ->setTitle(ucwords($_shippingMethod[0]))
                                    ->setPopup(1);

                                Mage::getModel('chronorelais/historyLt')->saveLtHistory($shipment->getId(), $expedition->return->skybillNumber, $dimensions[0]['weight']);
                                $shipment->addTrack($track);
                                $skybill = $expedition->return->pdfEtiquette;

                                $tracking_url = str_replace('{tracking_number}', $tracking_number,
                                    Mage::helper('chronorelais')->getConfigurationTrackingViewUrl());
                                $tracking_title = $this->__('Track Your Order');
                                $tracking_order = '<p><a title="' . $tracking_title . '" href="' . $tracking_url . '"><b>' . $tracking_title . '</b></a></p>';

                            } else {
                                $errorMessage = $_helper->__($expedition->return->errorMessage);
                                if($expedition->return->errorCode == 1) {
                                    $errorMessage = $_helper->__("Le contrat sélectionné pour la commande " . $_order->getIncrementId() . " ne prend pas en charge ce mode de transport.");
                                }
                                $this->_getSession()->addError($errorMessage);
                                return;
                            }
                        }
                    } catch (SoapFault $fault) {
                        $this->_getSession()->addError($_helper->__($fault->faultstring));

                        return;
                    }
                }

                $comment = '';
                $shipment->setEmailSent(true);
                $this->_saveShipment($shipment);
                $shipment->sendEmail(1, $tracking_order . $comment);
                $this->_getSession()->addSuccess($this->__('Shipment was successfully created.'));

                return $skybill;
            } else {
                $this->_forward('noRoute');

                return;
            }
        } catch (Mage_Core_Exception $e) {
            $this->_getSession()->addError($e->getMessage());

            return;
        } catch (Exception $e) {
            $this->_getSession()->addError($this->__('Can not save shipment: ' . $e->getMessage()));

            return;
        }
    }

    public function printMassAction() {
        /** @var Chronopost_Chronorelais_Helper_Data $helper */
        $helper = Mage::helper('chronorelais');
        $orderIds = $this->getRequest()->getParam('order_ids');
        $contractIds = explode('&', $this->getRequest()->getParam('contracts_ids'));
        $numbersParcels = explode('&', $this->getRequest()->getParam('numbers_parcels'));
        $allDimensions = urldecode($this->getRequest()->getParam('all_dimensions'));
        $allDimensions = preg_replace('/dimensions\=/', '', $allDimensions);
        $allDimensions = explode('&', $allDimensions);
        for ($i = 0; $i<count($allDimensions); $i++) {
            $allDimensions[$i] = json_decode($allDimensions[$i], true);
        }

        $skybillArr = [];
        foreach ($orderIds as $orderId) {
            $_shipments = $this->getShipmentByOrderId($orderId);
            if($_shipments){ // LTs Already generated
                foreach ($_shipments as $_shipment) {
                    $shipment = $shipment = Mage::getModel('sales/order_shipment')->getCollection()
                        ->addFieldToFilter('entity_id', $_shipment)
                        ->getFirstItem();
                    foreach ($_shipTracks = $shipment->getAllTracks() as $_shipTrack) {
                        if ($_shipTrack->getNumber() && $_shipTrack->getChronoReservationNumber()) {
                            $pdf = base64_decode($_shipTrack->getChronoReservationNumber());
                            if(!in_array($pdf, $skybillArr)) {
                                $skybillArr[] = $pdf;
                            }
                        }
                    }
                }
            } else {
                $index = -1;
                $contractId = null;
                foreach ($contractIds as $key => $_contract) {
                    preg_match('/select_contract_' . $orderId . '=(\-?[0-9]+)/', $_contract, $matches);
                    $contractId = ($matches[1] !== null) ? $matches[1] : null;
                    if($contractId != null){
                        $index = $key;
                        break;
                    }
                }

                $dimensions = $index != -1 ? $allDimensions[$index] : null;
                $numberParcel = preg_replace('/number_parcel=/', '', $numbersParcels[$index]);

                $order = Mage::getModel('sales/order')->load($orderId);

                /* If shipping method is Chronopost => check if shipping weight isn't over limit */
                $chronopostMethods = array(
                    'chronorelais',
                    'chronopost',
                    'chronoexpress',
                    'chronopostc10',
                    'chronopostc18',
                    'chronopostcclassic',
                    'chronorelaiseurope',
                    'chronorelaisdom',
                    'chronopostsrdv',
                    'chronopostsameday'
                );
                $shippingMethod = $order->getShippingMethod();
                $shippingMethod = explode("_", $shippingMethod);
                $shippingMethod = $shippingMethod[0];
                if (in_array($shippingMethod, $chronopostMethods)) {
                    $weightShipping = 0;
                    $weight_limit = Mage::getStoreConfig('carriers/' . $shippingMethod . '/weight_limit');
                    foreach ($order->getItemsCollection() as $item) {
                        $weightShipping += $item->getWeight() * $item->getQtyOrdered();
                    }
                    if ($helper->getConfigWeightUnit() == 'g') {
                        $weightShipping = $weightShipping / 1000; // conversion g => kg
                    }
                    if ($weightShipping > $weight_limit) {
                        /* multi shipping. 1 shipment by product */
                        foreach ($order->getItemsCollection() as $item) {
                            $qty = $item->getQtyOrdered();
                            for ($i = 1; $i <= $qty; $i++) {
                                $skybill = $this->createNewShipment($orderId, array($item->getId() => '1'), $contractId, $numberParcel, $dimensions);
                                if ($skybill) {
                                    $skybillArr[] = $skybill;
                                }
                            }
                        }
                    } else {
                        $skybill = $this->createNewShipment($orderId, '', $contractId, $numberParcel, $dimensions);
                        if ($skybill) {
                            $skybillArr[] = $skybill;
                        }
                    }
                } else {
                    $skybill = $this->createNewShipment($orderId, '', $contractId, $numberParcel, $dimensions);
                    if ($skybill) {
                        $skybillArr[] = $skybill;
                    }
                }
            }
        }

        if (count($skybillArr)) {
            $this->_processDownloadMass($skybillArr);
        } else {
            return $this->_redirectReferer();
        }
    }

    public function viewAction()
    {
        $shipmentId = $this->getRequest()->getParam('shipment_increment_id');
        $helper = Mage::helper('chronorelais');

        if($shipmentId) {

            $shipment = Mage::getModel('sales/order_shipment')->getCollection()
                ->addFieldToFilter('increment_id', $shipmentId)
                ->getFirstItem();

            $skybill = [];
            foreach ($_shipTracks = $shipment->getAllTracks() as $_shipTrack) {
                if ($_shipTrack->getNumber() && $_shipTrack->getChronoReservationNumber()) {
                    $pdf = base64_decode($_shipTrack->getChronoReservationNumber());
                    if(!in_array($pdf, $skybill)) {
                        $skybill[] = $pdf;
                    }
                    //break;
                }
            }
            $skybill = count($skybill) > 1 ? $skybill : $skybill[0];
            try {
                if (is_array($skybill)) {
                    $this->_processDownloadMass($skybill);
                } else {
                    $this->_prepareDownloadResponse('Etiquette_chronopost.pdf', $skybill);
                }
            } catch (Mage_Core_Exception $e) {
                $this->_getSession()->addError($helper->__('Désolé, une erreur est survenu lors de la récupération de l\'étiquettes. Merci de contacter Chronopost ou de réessayer plus tard'));
            }
        }
    }

    public function printAction()
    {
        if($view = $this->getRequest()->getParam('view') == 1) {
            return $this->viewAction();
        }

        $orderId = $this->getRequest()->getParam('order_id');
        $contractId = $this->getRequest()->getParam('contract_id');
        $number_parcel = $this->getRequest()->getParam('number_parcel');
        $dimensions = json_decode($this->getRequest()->getPost('dimensions'), true);

        $helper = Mage::helper('chronorelais');
        if ($orderId) {
            if ($_shipments = $this->getShipmentByOrderId($orderId)) {
                if($number_parcel != null && $number_parcel > 1) {
                    $shipmentId = $_shipments[0];
                    $skybillArr = $this->getEtiquetteMultiParcel($shipmentId, $contractId, $number_parcel, $dimensions);
                } else {
                    if (count($_shipments) == 1) {
                        $shipmentId = $_shipments[0];
                        $skybillArr = $this->getEtiquetteUrl($shipmentId, $contractId, $dimensions);
                    } else {
                        $track = "Cette commande contient plusieurs expéditions, cliquez sur chaque lien pour obtenir les étiquettes :<br>";
                        /*foreach ($_shipments as $_shipment) {
                            $url = str_replace('{trackingNumber}', $this->getEtiquetteUrl($_shipment), $helper->getConfigurationTrackingUrl());
                            $track .= '<a target="_blank" href="' . $url . '">' . $url . '</a><br />';
                        }*/
                        echo $track;

                        return;
                    }
                }
            } else {
                $order = Mage::getModel('sales/order')->load($orderId);

                /* If shipping method is Chronopost => check if shipping weight isn't over limit */
                $chronopostMethods = array(
                    'chronorelais',
                    'chronopost',
                    'chronoexpress',
                    'chronopostc10',
                    'chronopostc18',
                    'chronopostcclassic',
                    'chronorelaiseurope',
                    'chronorelaisdom',
                    'chronopostsrdv',
                    'chronopostsameday'
                );
                $shippingMethod = $order->getShippingMethod();
                $shippingMethod = explode("_", $shippingMethod);
                $shippingMethod = $shippingMethod[0];
                if (in_array($shippingMethod, $chronopostMethods)) {
                    $weightShipping = 0;
                    $shippingMethod = explode("_", $shippingMethod);
                    $shippingMethod = $shippingMethod[0];
                    $weight_limit = Mage::getStoreConfig('carriers/' . $shippingMethod . '/weight_limit');
                    foreach ($order->getItemsCollection() as $item) {
                        $weightShipping += $item->getWeight() * $item->getQtyOrdered();
                    }
                    if ($helper->getConfigWeightUnit() == 'g') {
                        $weightShipping = $weightShipping / 1000; // conversion g => kg
                    }
                    if ($weightShipping > $weight_limit) {
                        /* multi shipping. 1 shipment by product */
                        $skybillArr = array();
                        foreach ($order->getItemsCollection() as $item) {
                            $qty = $item->getQtyOrdered();
                            for ($i = 1; $i <= $qty; $i++) {
                                $skybillArr[] = $this->createNewShipment($orderId, array($item->getId() => '1'), $contractId, null, $dimensions);
                            }
                        }
                    } else {
                        if($number_parcel != null && $number_parcel > 1) {
                            $skybillArr = $this->createNewShipment($orderId, '', $contractId, $number_parcel, $dimensions);
                        } else {
                            $skybillArr = $this->createNewShipment($orderId, '', $contractId, null, $dimensions);
                        }
                    }
                } else {
                    if($number_parcel != null && $number_parcel > 1) {
                        $skybillArr = $this->createNewShipment($orderId, '', $contractId, $number_parcel, $dimensions);
                    } else {
                        $skybillArr = $this->createNewShipment($orderId, '', $contractId, null, $dimensions);
                    }
                }
            }
        } else {
            $shipmentId = $this->getRequest()->getPost('shipment_id');
            if ($shipmentId) {
                if($number_parcel != null && $number_parcel > 1) {
                    $skybillArr = $this->getEtiquetteMultiParcel($shipmentId, $contractId, $number_parcel, $dimensions);
                } else {
                    $skybillArr = $this->getEtiquetteUrl($shipmentId, $contractId, $dimensions);
                }
            } else {
                $shipmentIncrementId = $this->getRequest()->getParam('shipment_increment_id');
                $shipmentId = $this->getShipmentByIncrementId($shipmentIncrementId);
                if($number_parcel != null && $number_parcel > 1) {
                    $skybillArr = $this->getEtiquetteMultiParcel($shipmentId[0], $contractId, $number_parcel, $dimensions);
                } else {
                    $skybillArr = $this->getEtiquetteUrl($shipmentId[0], $contractId, $dimensions);
                }
            }
        }

        if ($skybillArr) {
            try {
                if (is_array($skybillArr)) {
                    $this->_processDownloadMass($skybillArr);
                } else {
                    $this->_prepareDownloadResponse('Etiquette_chronopost.pdf', $skybillArr);
                }
                $this->_redirectReferer();
            } catch (Mage_Core_Exception $e) {
                $this->_getSession()->addError($helper->__('Désolé, une erreur est survenu lors de la récupération de l\'étiquettes. Merci de contacter Chronopost ou de réessayer plus tard'));
            }
        } else {
            return $this->_redirectReferer();
        }
    }

    public function massLivraisonSamediStatusAction()
    {
        if ($this->getRequest()->getPost('status')) {
            $this->saveLivraisonSamediStatusAction();
        }
    }

    /* Save the Livraison le Samedi status to orders */

    public function saveLivraisonSamediStatusAction()
    {
        /* get the orders */
        $orderIds = $this->getRequest()->getPost('order_ids');
        $status = $this->getRequest()->getPost('status');
        $_connection = Mage::getSingleton('core/resource')->getConnection('core_write');
        $_table = Mage::getSingleton('core/resource')->getTableName('sales_chronopost_order_export_status');
        $exceptions = array();

        foreach ($orderIds as $orderId) {
            $order_details = Mage::getModel('sales/order')->load($orderId);
            $shipping_method = '';
            $livraison_le_samedi = $status;
            if ($shipping_method = $order_details->getShippingMethod()) {
                $shipping_method = explode('_', $shipping_method);
                if ($shipping_method[0] == 'chronoexpress') {
                    $livraison_le_samedi = '--';
                }
            }
            $condition = array(
                $_connection->quoteInto('order_id = ?', $orderId),
            );
            $_connection->delete($_table, $condition);

            $dataLine = array(
                'order_id'            => $orderId,
                'livraison_le_samedi' => $livraison_le_samedi
            );
            try {
                $_connection->insert($_table, $dataLine);
            } catch (Exception $e) {
                $exceptions[] = Mage::helper('chronorelais')->__('Order assigning error: ' . $e->getMessage());
            }
        }
        if ($exceptions) {
            $this->_getSession()->addError($exceptions);
        } else {
            $this->_getSession()->addSuccess($this->__('Livraison le Samedi statut a &eacute;t&eacute; ajout&eacute;'));
        }
        $this->_redirect('*/*/index');
    }

    /* Remove accents characters */

    public function removeaccents($string)
    {
        $stringToReturn = str_replace(
            array(
                'à',
                'á',
                'â',
                'ã',
                'ä',
                'ç',
                'è',
                'é',
                'ê',
                'ë',
                'ì',
                'í',
                'î',
                'ï',
                'ñ',
                'ò',
                'ó',
                'ô',
                'õ',
                'ö',
                'ù',
                'ú',
                'û',
                'ü',
                'ý',
                'ÿ',
                'À',
                'Á',
                'Â',
                'Ã',
                'Ä',
                'Ç',
                'È',
                'É',
                'Ê',
                'Ë',
                'Ì',
                'Í',
                'Î',
                'Ï',
                'Ñ',
                'Ò',
                'Ó',
                'Ô',
                'Õ',
                'Ö',
                'Ù',
                'Ú',
                'Û',
                'Ü',
                'Ý',
                '/',
                '\xa8'
            ), array(
            'a',
            'a',
            'a',
            'a',
            'a',
            'c',
            'e',
            'e',
            'e',
            'e',
            'i',
            'i',
            'i',
            'i',
            'n',
            'o',
            'o',
            'o',
            'o',
            'o',
            'u',
            'u',
            'u',
            'u',
            'y',
            'y',
            'A',
            'A',
            'A',
            'A',
            'A',
            'C',
            'E',
            'E',
            'E',
            'E',
            'I',
            'I',
            'I',
            'I',
            'N',
            'O',
            'O',
            'O',
            'O',
            'O',
            'U',
            'U',
            'U',
            'U',
            'Y',
            ' ',
            'e'
        ), $string);
        // Remove all remaining other unknown characters
        $stringToReturn = preg_replace('/[^a-zA-Z0-9\-]/', ' ', $stringToReturn);
        $stringToReturn = preg_replace('/^[\-]+/', '', $stringToReturn);
        $stringToReturn = preg_replace('/[\-]+$/', '', $stringToReturn);
        $stringToReturn = preg_replace('/[\-]{2,}/', ' ', $stringToReturn);

        return $stringToReturn;
    }

    /*
     * *******************************************************************
     * ******************** ETIQUETTE DE RETOUR **************************
     * *******************************************************************
     */

    public function printEtiquetteRetourAction()
    {
        $shipmentIncrementId = $this->getRequest()->getParam('shipment_increment_id');
        $shipmentId = $this->getShipmentByIncrementId($shipmentIncrementId);
        $shipmentId = $shipmentId[0];
        $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);
        $_order = $shipment->getOrder();
        $_shippingAddress = $shipment->getShippingAddress();
        $_billingAddress = $shipment->getBillingAddress();
        $skillbill = $this->getEtiquetteRetourUrl($shipment); /* skillbill pdf encode base64*/

        if ($skillbill) {
            try {
                $path = $this->savePdfWithContent($skillbill, $shipmentId);
                $message_email = 'Bonjour,
                                <br />Vous allez bientôt effectuer un envoi Chronopost. La personne qui vous a adressé ce mail a déjà préparé la lettre de transport que vous utiliserez. Après impression, apposez la lettre de transport dans une pochette plastique adhésive et collez la sur votre envoi. Attention le code à barres doit être bien apparent.
                                <br />Cordialement,';

                $customer_email = ($_shippingAddress->getEmail()) ? $_shippingAddress->getEmail() : ($_billingAddress->getEmail() ? $_billingAddress->getEmail() : $_order->getCustomerEmail());

                $mail = new Zend_Mail('utf-8');
                $mail->setBodyHtml($message_email);
                $mail->setFrom(Mage::getStoreConfig('contacts/email/recipient_email'));
                $mail->setSubject($_order->getStoreName(1) . ' : Etiquette de retour chronopost');
                $mail->createAttachment(file_get_contents($path), Zend_Mime::TYPE_OCTETSTREAM,
                    Zend_Mime::DISPOSITION_ATTACHMENT, Zend_Mime::ENCODING_BASE64, 'etiquette_retour.pdf');

                $mail->addTo($customer_email);
                $mail->send();

                $mail->clearRecipients();
                $mail->addTo(Mage::getStoreConfig('contacts/email/recipient_email'));
                $mail->send();

                $this->_getSession()->addSuccess(Mage::helper('chronorelais')->__('L\'etiquette de retour à bien été envoyée au client.'));
            } catch (Mage_Core_Exception $e) {
                $this->_getSession()->addError(Mage::helper('chronorelais')->__('Désolé, une erreure est survenu lors de la récupération de l\'étiquettes. Merci de contacter Chronopost ou de réessayer plus tard'));
            }
        }

        return $this->_redirectReferer();
    }

    protected function getEtiquetteRetourParams($shipment, $_shippingMethod)
    {
        $_order = $shipment->getOrder();
        $_shippingAddress = $shipment->getShippingAddress();
        $_billingAddress = $shipment->getBillingAddress();

        /**
         * @var Chronopost_Chronorelais_Helper_Data $_helper
         */
        $_helper = Mage::helper('chronorelais');

        $contract = $_helper->getContractOfOrder($_order->getId());
        $accountNumber = $contract["contract_account_number"];
        $subAccount = $contract["contract_account_sub_number"];
        $pass = $contract["contract_account_password"];


        $shippingMethodAllow = array(
            'chronorelaiseurope',
            'chronorelais',
            'chronopost',
            'chronopostc10',
            'chronopostc18',
            'chronopostsrdv',
            'chronoexpress',
            'chronopostcclassic'
        );
        if (!in_array($_shippingMethod[0], $shippingMethodAllow)) {
            $this->_getSession()->addError($_helper->__('Les retours ne sont pas disponibles pour le mode de livraison ' . $_shippingMethod[0]));

            return;
        }

        if (in_array($_shippingMethod[0], $shippingMethodAllow)) {
            $esdParams = $header = $shipper = $customer = $recipient = $ref = $skybill = $skybillParams = $password = array();

            //header parameters

            $header = array(
                'idEmit'        => 'MAG',
                'accountNumber' => $accountNumber,
                'subAccount'    => $subAccount
            );


            //shipper parameters
            $recipient = $this->getRecipientReturnAdress();

            //customer parameters
            $customerMobilePhone = $this->checkMobileNumber($_helper->getConfigurationCustomerInfo('mobilephone'));
            $customer = array(
                'customerAdress1'     => $_helper->getConfigurationCustomerInfo('address1'),
                'customerAdress2'     => $_helper->getConfigurationCustomerInfo('address2'),
                'customerCity'        => $_helper->getConfigurationCustomerInfo('city'),
                'customerCivility'    => $_helper->getConfigurationCustomerInfo('civility'),
                'customerContactName' => $_helper->getConfigurationCustomerInfo('contactname'),
                'customerCountry'     => $_helper->getConfigurationCustomerInfo('country'),
                'customerEmail'       => $_helper->getConfigurationCustomerInfo('email'),
                'customerMobilePhone' => $customerMobilePhone,
                'customerName'        => $_helper->getConfigurationCustomerInfo('name'),
                'customerName2'       => $_helper->getConfigurationCustomerInfo('name2'),
                'customerPhone'       => $_helper->getConfigurationCustomerInfo('phone'),
                'customerPreAlert'    => '',
                'customerZipCode'     => $_helper->getConfigurationCustomerInfo('zipcode')
            );

            //recipient parameters
            $_recipientAddress = $_shippingAddress;
            if (strpos($_shippingMethod[0], 'chronorelais') !== false) {
                // Nicolas, le 27/11/2014 : si Chronorelais, on doit utiliser l'adresse de facturation, non de livraison (qui est celle du relais)
                $_recipientAddress = $_billingAddress;
            }
            $recipient_address = $_recipientAddress->getStreet();

            // Champs forcément basés sur l'adresse de livraison
            $customer_email = ($_shippingAddress->getEmail()) ? $_shippingAddress->getEmail() : ($_billingAddress->getEmail() ? $_billingAddress->getEmail() : $_order->getCustomerEmail());
            $recipientMobilePhone = $this->checkMobileNumber($_shippingAddress->getTelephone());
            $recipientName = $this->getFilledValue($_recipientAddress->getCompany()); //RelayPoint Name if chronorelais or Companyname if chronopost and
            $recipientName2 = $this->getFilledValue($_shippingAddress->getFirstname() . ' ' . $_shippingAddress->getLastname());
            //remove any alphabets in phone number

            $recipientPhone = trim(preg_replace("/[^0-9\.\-]/", " ", $_shippingAddress->getTelephone()));
            if (!isset($recipient_address[1])) {
                $recipient_address[1] = '';
            }

            $shipper = array(
                'shipperAdress1'     => substr($this->getFilledValue($recipient_address[0]), 0, 38),
                'shipperAdress2'     => $recipient_address[1] ? substr($this->getFilledValue($recipient_address[1]), 0,
                    38) : '',
                'shipperCity'        => $this->getFilledValue($_recipientAddress->getCity()),
                'shipperCivility'    => 'M',
                'shipperContactName' => $recipientName2,
                'shipperCountry'     => $this->getFilledValue($_recipientAddress->getCountryId()),
                'shipperEmail'       => $customer_email,
                'shipperMobilePhone' => $recipientMobilePhone,
                'shipperName'        => $recipientName,
                'shipperName2'       => $recipientName2,
                'shipperPhone'       => $recipientPhone,
                'shipperPreAlert'    => '',
                'shipperZipCode'     => $this->getFilledValue($_recipientAddress->getPostcode()),
            );

            //ref parameters
            $recipientRef = $this->getFilledValue($_shippingAddress->getWRelayPointCode());
            if (!$recipientRef) {
                $recipientRef = $_order->getCustomerId();
            }
            $shipperRef = $_order->getRealOrderId();

            $ref = array(
                'recipientRef' => $recipientRef,
                'shipperRef'   => $shipperRef
            );

            //skybill parameters
            /* Livraison Samedi (Delivery Saturday) field */
            $SaturdayShipping = 0; //default value for the saturday shipping
            if ($_shippingMethod[0] == "chronopost" || $_shippingMethod[0] == "chronorelais") {
                if (!$_deliver_on_saturday = Mage::helper('chronorelais')->getLivraisonSamediStatus($_order->getEntityId())) {
                    $_deliver_on_saturday = Mage::helper('chronorelais')->getConfigData('carriers/' . $_shippingMethod[0] . '/deliver_on_saturday');
                } else {
                    if ($_deliver_on_saturday == 'Yes') {
                        $_deliver_on_saturday = 1;
                    } else {
                        $_deliver_on_saturday = 0;
                    }
                }
                $is_sending_day = Mage::helper('chronorelais')->isSendingDay();
                if ($_deliver_on_saturday && $is_sending_day) {
                    $SaturdayShipping = 6;
                } elseif (!$_deliver_on_saturday && $is_sending_day) {
                    $SaturdayShipping = 1;
                }
            }

            $weight = 0;
            foreach ($shipment->getItemsCollection() as $item) {
                $weight += $item->weight * $item->qty;
            }
            if ($_helper->getConfigWeightUnit() == 'g') {
                $weight = $weight / 1000; /* conversion g => kg */
            }

            $productCode = $_helper->getReturnProductCode($_shippingAddress, $contract);
//            if($_shippingMethod[0] == 'chronorelaiseurope') {
//                $productCode = '3T';
//                //$weight <= 3 ? $SaturdayShipping = '337' : $SaturdayShipping = '338';
//                $SaturdayShipping = '332';
//            }

            // @todo réactiver ce code quand les WS auront été mis à jour
            //$codeService = $_helper->getReturnServiceCode($productCode);

            if($recipient['recipientCountry'] == 'FR' && $shipper['shipperCountry'] != 'FR') {
                $productCode = '3T';
            }

            if ($SaturdayShipping == 6) {
                $codeService = 1;
            } else {
                $codeService = 0;
            }

            if ($_shippingMethod[0] == "chronopostdimanche") {
                $codeService = Chronopost_Chronorelais_Helper_Data::CHRONOPOST_DIM_BAL_SERVICE;
            }

            $weight = 0; /* On met le poids à 0 car les colis sont pesé sur place */

            $skybill = array(
                'codCurrency'     => 'EUR',
                'codValue'        => '',
                'content1'        => '',
                'content2'        => '',
                'content3'        => '',
                'content4'        => '',
                'content5'        => '',
                'customsCurrency' => 'EUR',
                'customsValue'    => '',
                'evtCode'         => 'DC',
                'insuredCurrency' => 'EUR',
                'insuredValue'    => '',
                'objectType'      => 'MAR',
                'productCode'     => $productCode,
                'service'         => $codeService,
                'shipDate'        => date('c'),
                'shipHour'        => date('H'),
                'weight'          => $weight,
                'weightUnit'      => 'KGM',
                'height'          => 1,
                'length'          => 1,
                'width'           => 1
            );

            $mode = $_helper->getConfigurationSkybillParam();
            if ($_shippingMethod[0] == 'chronorelaiseurope') {
                $mode = 'PPR';
            }
            $skybillParams = array(
                'mode' => $mode,
                'withReservation' => 0
            );

            $expeditionArray = array(
                'headerValue'        => $header,
                'shipperValue'       => $shipper,
                'customerValue'      => $customer,
                'recipientValue'     => $recipient,
                'refValue'           => $ref,
                'skybillValue'       => $skybill,
                'skybillParamsValue' => $skybillParams,
                'password'           => $pass
            );

            return $expeditionArray;
        }
    }

    protected function getRecipientReturnAdress()
    {
        if ($return_adress = $this->getRequest()->getParam('recipient_adress')) {
            switch ($return_adress) {
                case "shipper_adress":
                    return $this->getConfigShipperAdress();
                    break;
                case "customer_adress":
                    return $this->getConfigCustomerAdress();
                    break;
                case "return_adress":
                    return $this->getConfigReturnAdress();
                    break;
                default :
                    return $this->getConfigShipperAdress();
                    break;
            }
        } else {
            return $this->getConfigShipperAdress();
        }


    }

    private function getConfigCustomerAdress()
    {
        $_helper = Mage::helper('chronorelais');
        $customerMobilePhone = $this->checkMobileNumber($_helper->getConfigurationCustomerInfo('mobilephone'));

        return array(
            'recipientAdress1'     => $_helper->getConfigurationCustomerInfo('address1'),
            'recipientAdress2'     => $_helper->getConfigurationCustomerInfo('address2'),
            'recipientCity'        => $_helper->getConfigurationCustomerInfo('city'),
            'recipientCivility'    => $_helper->getConfigurationCustomerInfo('civility'),
            'recipientContactName' => $_helper->getConfigurationCustomerInfo('contactname'),
            'recipientCountry'     => $_helper->getConfigurationCustomerInfo('country'),
            'recipientEmail'       => $_helper->getConfigurationCustomerInfo('email'),
            'recipientMobilePhone' => $customerMobilePhone,
            'recipientName'        => $_helper->getConfigurationCustomerInfo('name'),
            'recipientName2'       => $_helper->getConfigurationCustomerInfo('name2'),
            'recipientPhone'       => $_helper->getConfigurationCustomerInfo('phone'),
            'recipientPreAlert'    => '',
            'recipientZipCode'     => $_helper->getConfigurationCustomerInfo('zipcode')
        );
    }

    private function getConfigShipperAdress()
    {
        $_helper = Mage::helper('chronorelais');
        $shipperMobilePhone = $this->checkMobileNumber($_helper->getConfigurationShipperInfo('mobilephone'));

        return array(
            'recipientAdress1'     => $_helper->getConfigurationShipperInfo('address1'),
            'recipientAdress2'     => $_helper->getConfigurationShipperInfo('address2'),
            'recipientCity'        => $_helper->getConfigurationShipperInfo('city'),
            'recipientCivility'    => $_helper->getConfigurationShipperInfo('civility'),
            'recipientContactName' => $_helper->getConfigurationShipperInfo('contactname'),
            'recipientCountry'     => $_helper->getConfigurationShipperInfo('country'),
            'recipientEmail'       => $_helper->getConfigurationShipperInfo('email'),
            'recipientMobilePhone' => $shipperMobilePhone,
            'recipientName'        => $_helper->getConfigurationShipperInfo('name'),
            'recipientName2'       => $_helper->getConfigurationShipperInfo('name2'),
            'recipientPhone'       => $_helper->getConfigurationShipperInfo('phone'),
            'recipientPreAlert'    => '',
            'recipientZipCode'     => $_helper->getConfigurationShipperInfo('zipcode')
        );
    }

    private function getConfigReturnAdress()
    {
        $_helper = Mage::helper('chronorelais');
        $returnMobilePhone = $this->checkMobileNumber($_helper->getConfigurationReturnInfo('mobilephone'));

        return array(
            'recipientAdress1'     => $_helper->getConfigurationReturnInfo('address1'),
            'recipientAdress2'     => $_helper->getConfigurationReturnInfo('address2'),
            'recipientCity'        => $_helper->getConfigurationReturnInfo('city'),
            'recipientCivility'    => $_helper->getConfigurationReturnInfo('civility'),
            'recipientContactName' => $_helper->getConfigurationReturnInfo('contactname'),
            'recipientCountry'     => $_helper->getConfigurationReturnInfo('country'),
            'recipientEmail'       => $_helper->getConfigurationReturnInfo('email'),
            'recipientMobilePhone' => $returnMobilePhone,
            'recipientName'        => $_helper->getConfigurationReturnInfo('name'),
            'recipientName2'       => $_helper->getConfigurationReturnInfo('name2'),
            'recipientPhone'       => $_helper->getConfigurationReturnInfo('phone'),
            'recipientPreAlert'    => '',
            'recipientZipCode'     => $_helper->getConfigurationReturnInfo('zipcode')
        );
    }


    protected function getEtiquetteRetourUrl($shipment)
    {
        //On récupère les infos d'expédition
        /** @var Chronopost_Chronorelais_Helper_Data $_helper */
        $_helper = Mage::helper('chronorelais');

        $_order = $shipment->getOrder();
        $_shippingMethod = explode("_", $_order->getShippingMethod());
        $_shippingAddress = $shipment->getShippingAddress();
        $countryId = $_shippingAddress->getCountryId();
        if(!$_helper->returnAuthorized($countryId)) {
            $this->_getSession()->addError($_helper->__("Les étiquettes de retour ne sont pas disponibles pour ce pays : " . $countryId));
            return false;
        }

        $expeditionArray = $this->getEtiquetteRetourParams($shipment, $_shippingMethod);
        if ($expeditionArray) {
            $client = new SoapClient("https://www.chronopost.fr/shipping-cxf/ShippingServiceWS?wsdl",
                array('trace' => true));
            try {
                $webservbt = $client->shippingV7($expeditionArray);

                if (!$webservbt->return->errorCode && $webservbt->return->pdfEtiquette) {
                    return $webservbt->return->pdfEtiquette;
                } else {
                    if (isset($webservbt->return->errorMessage)) {
                        $errorMessage = $_helper->__($webservbt->return->errorMessage);
                        if($webservbt->return->errorCode == 1) {
                            $errorMessage = $_helper->__("Le colis de la commande " . $_order->getIncrementId() . " n'appartient pas au contrat passé en paramètre.");
                        }
                        $this->_getSession()->addError($errorMessage);
                    } else {
                        $this->_getSession()->addError($_helper->__('An error occured while contacting Chronopost webservice. Please try again later.'));
                    }
                }
            } catch (SoapFault $fault) {
                $this->_getSession()->addError($_helper->__($fault->faultstring));
            }
        }
    }

    protected function savePdf($url, $shipmentId)
    {
        $this->createMediaChronopostFolder();
        $path = 'media/chronopost/etiquetteRetour-' . $shipmentId . '.pdf';
        file_put_contents($path, file_get_contents($url));

        return $path;
    }

    protected function savePdfWithContent($content_base64, $shipmentId)
    {
        $this->createMediaChronopostFolder();
        $path = 'media/chronopost/etiquetteRetour-' . $shipmentId . '.pdf';
        file_put_contents($path, $content_base64);

        return $path;
    }

    /* create folder media/chronopost if not exist */
    protected function createMediaChronopostFolder()
    {
        $path = 'media/chronopost';
        if (!is_dir($path)) {
            mkdir($path, 0777);
        }
    }

    protected function getShipmentObjectByOrderId($orderId)
    {
        $_shipment = Mage::getResourceModel('sales/order_shipment_collection')
            ->addAttributeToFilter('order_id', $orderId);

        return $_shipment;
    }

    public function cancelMassAction()
    {
        /** @var Chronopost_Chronorelais_Helper_Data $_helper */
        $_helper = Mage::helper('chronorelais');
        $orderIds = $this->getRequest()->getParam('order_ids');
        $contractIds = $this->getRequest()->getParam('contracts_ids');

        if ($orderIds) {
            $nbEtiquettesDelete = 0;
            foreach ($orderIds as $orderId) {
                $shipments = $this->getShipmentObjectByOrderId($orderId);
                preg_match('/select_contract_' . $orderId . '=(\-?[0-9]+)/', $contractIds, $matches);
                $contractId = ($matches[1] !== null) ? $matches[1] : null;
                if ($shipments) {
                    $associatedContractDeleted = false;
                    foreach ($shipments as $shipment) {

                        $tracks = $shipment->getTracksCollection();
                        foreach ($tracks as $track) {

                            /* numero chrono si getChronoReservationNumber non null */
                            if ($track->getChronoReservationNumber()) {

                                /* appel WS pour annuler LT */
                                if($contractId < 0) {
                                    $contract = $_helper->getContractOfOrder($orderId);
                                    $webservbt = Mage::helper('chronorelais/webservice')->cancelSkybill($track->getNumber(), $contract);
                                } else {
                                    $webservbt = Mage::helper('chronorelais/webservice')->cancelSkybill($track->getNumber(), $contractId);
                                }

                                if ($webservbt) {
                                    /* suppression du numéro de tracking */
                                    if ($webservbt->return->errorCode == 0) {
                                        if(!$associatedContractDeleted) {
                                            $_helper->deleteContractOfOrder($orderId);
                                            $associatedContractDeleted = true;
                                        }
                                        $nbEtiquettesDelete++;
                                        $track->delete();
                                    } else {
                                        switch ($webservbt->return->errorCode) {
                                            case "1" :
                                                $errorMessage = $this->__("Une erreur système est survenue");
                                                break;
                                            case "2" :
                                                $errorMessage = $this->__("le colis n'appartient pas au contrat passé en paramètre ou n'a pas encore été enregistré dans le système de tracking Chronopost");
                                                break;
                                            case "3" :
                                                $errorMessage = $this->__("Le colis ne peut être pas annulé car il a été pris en charge par Chronopost");
                                                break;
                                            default :
                                                $errorMessage = '';
                                                break;
                                        }
                                        $this->_getSession()->addError($this->__("Erreur lors de la suppression de l'étiquette %s : %s.",
                                            $track->getNumber(), $errorMessage));
                                    }
                                } else {
                                    $this->_getSession()->addError($this->__('Désolé, une erreur est survenu lors de la suppression de l\'étiquette %s. Merci de contacter Chronopost ou de réessayer plus tard',
                                        $track->getNumber()));
                                }
                            }
                        }
                    }
                }
            }
            if ($nbEtiquettesDelete > 0) {
                if ($nbEtiquettesDelete > 1) {
                    $this->_getSession()->addSuccess($this->__('%s étiquettes de transport ont bien été annulées.',
                        $nbEtiquettesDelete));
                } else {
                    $this->_getSession()->addSuccess($this->__('%s étiquette de transport a bien été annulée.',
                        $nbEtiquettesDelete));
                }

            }
        } else {
            $this->_getSession()->addError($this->__("Veuillez sélectionner au moins une lettre de transport."));
        }
        $this->_redirect('*/*/index');
    }

}
