Event.observe(window, 'load', function () {
    buildDimensionsJson();

    function onInputDimensionsKeyUp(pos, e) {
        if (isNaN(parseInt(e.key))) {
            e.target.value = e.target.value.replace(/\D/g, '');
        }
        var type = e.target.up().readAttribute('class');
        correctDimensions();
        buildDimensionsJson();
    }

    $$('#dimensions-weight .dimensions-input-container input').forEach(function (input) {
        input.on('keyup', function (e) {
            onInputDimensionsKeyUp(input.readAttribute('position'), e);
        });
    });

    $('nb_colis').on('keyup', function (e) {
        if (isNaN(parseInt(e.key))) {
            e.target.value = e.target.value.replace(/\D/g, '');
            if (!isNaN(e.target.value) || e.target.value !== '') {
                if (e.target.value == 0) {
                    updateNbInputs(1, orderWeight);
                } else {
                    updateNbInputs(e.target.value, orderWeight);
                }
            }
        } else {
            if (e.target.value > 100) {
                e.target.value = 100;
            }
            if (e.target.value !== '') {
                if (e.target.value == 0) {
                    updateNbInputs(1, orderWeight);
                } else {
                    updateNbInputs(e.target.value, orderWeight);
                }
            }
        }
        checkAllcorrectDimensions();
    });

    function checkAllcorrectDimensions() {
        while($('error-inputs').lastElementChild) {
            $('error-inputs').lastElementChild.remove();
        }
        var rows = $$('#dimensions-weight .dimensions-input-container');
        var i;
        for (i = 0; i<rows.length; i++) {
            correctDimensions(i+1);
        }
    }

    function updateNbInputs(nbAsked, weight) {
        var tbody = $$('#dimensions-weight tbody');
        var rows = $$('#dimensions-weight .dimensions-input-container');
        var nbRow = rows.length;
        var lastIndex = 0;
        if (rows[0].down('input')) {
            lastIndex = rows[rows.length - 1].down('input').readAttribute('position');
        }

        if (nbAsked > nbRow) {
            for (var j = 0; j < (nbAsked - nbRow); j++) {
                tbody[0].appendChild(createTr(lastIndex, weight));
                lastIndex++;
            }
        } else if (nbAsked < nbRow) {
            var cpt = nbRow - nbAsked;
            while (cpt > 0 && tbody[0].lastElementChild) {
                tbody[0].lastElementChild.remove();
                cpt--;
            }
        }
        buildDimensionsJson();
    }

    function createTr(pos, weight) {
        var tr = document.createElement('tr');
        tr.setAttribute('class', 'dimensions-input-container');
        var td_weight = document.createElement('td');
        var td_width = document.createElement('td');
        var td_height = document.createElement('td');
        var td_length = document.createElement('td');
        td_weight.setAttribute('class', 'weight_input');
        td_width.setAttribute('class', 'width_input');
        td_height.setAttribute('class', 'height_input');
        td_length.setAttribute('class', 'length_input');
        td_weight.appendChild(createInput(td_weight, pos, weight));
        td_width.appendChild(createInput(td_width, pos, weight));
        td_height.appendChild(createInput(td_height, pos, weight));
        td_length.appendChild(createInput(td_length, pos, weight));
        tr.appendChild(td_weight);
        tr.appendChild(td_width);
        tr.appendChild(td_height);
        tr.appendChild(td_length);
        return tr;
    }

    function createInput(td, pos, weight) {
        var input = document.createElement('input');
        if (td.readAttribute('class') === 'weight_input') {
            input.setAttribute('value', weight);
        } else {
            input.setAttribute('value', '1');
        }
        input.setAttribute('class', 'input-text');
        input.setAttribute('style', 'margin-bottom:5px;width:75px;margin-right:5px');
        input.setAttribute('position', parseInt(pos) + 1 + '');
        input.on('keyup', function (e) {
            onInputDimensionsKeyUp(input.readAttribute('position'), e)
        });
        return input;
    }

    function buildDimensionsJson() {
        var dimensions = {};
        var trs = $$('#dimensions-weight tbody .dimensions-input-container');

        for (var i = 0; i < trs.length; i++) {
            var dimension = {};
            dimension.weight = trs[i].down('.weight_input').down('input').value;
            dimension.width = trs[i].down('.width_input').down('input').value;
            dimension.height = trs[i].down('.height_input').down('input').value;
            dimension.length = trs[i].down('.length_input').down('input').value;
            dimensions[i] = dimension;
        }
        $('input_dimensions').setAttribute('value', JSON.stringify(dimensions));
    }

    function correctDimensions() {
        var allInputs = $$('tr.dimensions-input-container input');
        var error = false;
        var errorLimitWeight = checkWeightLimit(allInputs);
        var errorDimensions = checkDimensionsLimit(allInputs);
        var errorGlobalDimensions = checkGlobalDimensions();
        if(errorLimitWeight) {
            addErrorMessage('weight');
            error = true;
        } else {
            removeErrorMessage('weight');
            error = false;
        }

        if(errorDimensions) {
            addErrorMessage('length');
            error = true;
        } else {
            removeErrorMessage('length');
            error = false;
        }

        if(errorGlobalDimensions) {
            addErrorMessage('global');
            error = true;
        } else {
            removeErrorMessage('global');
            error = false;
        }

        return error;
    }

    function checkWeightLimit(inputs) {
        var error = false;
        inputs.forEach(function(input) {
            if(input.up().readAttribute('class') === 'weight_input') {
                if (input.value > limitWeight) {
                    error = true;
                }
            }
        });
        return error;
    }

    function checkGlobalDimensions() {
        var error = false;
        var containers = $$('.dimensions-input-container');
        var totalDimensions;
        containers.forEach(function (container) {
            totalDimensions = 0;
            var inputs = container.querySelectorAll('input');
            inputs.forEach(function(input) {
                if(input.up().readAttribute('class') !== 'weight_input') {
                    if (input.up().readAttribute('class') === 'width_input') {
                        totalDimensions += parseInt(input.value);
                    } else {
                        totalDimensions += 2 * parseInt(input.value);
                    }
                }
            });
            if (totalDimensions > limitTotalDimensions) {
                error = true;
            }
        });

        return error;
    }

    function checkDimensionsLimit(inputs) {
        var error = false;
        inputs.forEach(function(input) {
            if(input.up().readAttribute('class') !== 'weight_input') {
                if (input.value > limitInputsDimensions) {
                    error = true;
                }
            }
        });

        return error;
    }

    function removeErrorMessage(type) {
        var div = $('error-' + type);
        if (div) {
            div.remove();
        }
        if($('error-inputs').childElementCount === 0) {
            var submit_button = $$('.order-totals-bottom button')[0];
            submit_button.enable();
        }
    }

    function addErrorMessage(typeError) {
        var container_error = $('error-inputs');
        if (!$('error-' + typeError)) {
            var div = document.createElement('div');
            div.setAttribute('id', 'error-'+ typeError);
            div.setAttribute('style', 'margin-top:5px;color:red;font-weight:bold');
            var errorMessage;
            switch (typeError) {
                case "weight":
                    errorMessage = Translator.translate('One or several packages are above the weight limit (%s kg)').replace('%s', limitWeight);
                    break;
                case "length" :
                    errorMessage = Translator.translate('One or several packages are above the size limit (%s cm)').replace('%s', limitInputsDimensions);
                    break;
                case "global" :
                    errorMessage = Translator.translate('One or several packages are above the total (L+2H+2l) size limit (%s cm)').replace('%s', limitTotalDimensions);
                    break;
            }
            var text = document.createTextNode(errorMessage);
            div.appendChild(text);
            container_error.appendChild(div);
        }
        var submit_button = $$('.order-totals-bottom button')[0];
        submit_button.disable();
    }

});
